import 'package:uuid_type/uuid_type.dart';

class UniqueId {
  final Uuid value;

  /// Generates a version-4 unique uid
  factory UniqueId() {
    return UniqueId._(Uuid(uuid.v4()));
  }

  /// Converts a unique uid string to unique uid type
  factory UniqueId.fromUniqueString(String uniqueId) {
    assert(uniqueId != null, 'uniqueId cannot be null');
    return UniqueId._(Uuid(uniqueId));
  }

  /// Serializes a unique uid string from a `Map` object with the below structure to unique uid type
  ///
  /// Example structure:
  ///
  /// `
  /// {
  ///   'id': 'f9eb93fe-07d9-4b38-8117-f709aa438a18'
  /// }
  /// `
  ///
  factory UniqueId.fromJson(Map<String, dynamic> json) {
    assert(json != null && json.isNotEmpty,
        'UniqueId.fromJson cannot be null or empty');
    return UniqueId._(Uuid(json['id']));
  }

  const UniqueId._(this.value);

  @override
  bool operator ==(Object o) {
    if (identical(this, o)) return true;

    return o is UniqueId && o.value == value;
  }

  @override
  int get hashCode => value.hashCode;

  @override
  String toString() => 'UniqueId(value: ${value.toString()})';
}
